@@ -1,11 +1,12 @@
 # Copyright Yahoo. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 
+import os
 import sys
 import ssl
 import aiohttp
 import asyncio
 import concurrent.futures
-
+from collections import Counter
 from typing import Optional, Dict, Tuple, List, IO, Union
 
 import requests
@@ -19,9 +20,7 @@ from tenacity import retry, wait_exponential, stop_after_attempt
 from time import sleep
 
 from vespa.io import VespaQueryResponse, VespaResponse
-from vespa.ml import TextTask
 from vespa.query import QueryModel
-from vespa.evaluation import EvalMetric
 from vespa.package import ApplicationPackage
 
 retry_strategy = Retry(
@@ -63,25 +62,26 @@ def parse_labeled_data(df):
     return labeled_data
 
 
-def parse_feed_df(df: DataFrame, include_id):
+def parse_feed_df(df: DataFrame, include_id, id_field="id"):
     """
     Convert a df into batch format for feeding
 
     :param df: DataFrame with the following required columns ["id"]. Additional columns are assumed to be fields.
     :param include_id: Include id on the fields to be fed.
+    :param id_field: Name of the column containing the id field.
     :return: List of Dict containing 'id' and 'fields'.
     """
-    required_columns = ["id"]
+    required_columns = [id_field]
     assert all(
         [x in list(df.columns) for x in required_columns]
     ), "DataFrame needs at least the following columns: {}".format(required_columns)
     records = df.to_dict(orient="records")
     batch = [
         {
-            "id": record["id"],
+            "id": record[id_field],
             "fields": record
             if include_id
-            else {k: v for k, v in record.items() if k not in ["id"]},
+            else {k: v for k, v in record.items() if k not in [id_field]},
         }
         for record in records
     ]
@@ -341,6 +341,7 @@ class Vespa(object):
                 **kwargs,
             )
 
+    @retry(wait=wait_exponential(multiplier=1), stop=stop_after_attempt(3))
     def query_batch(
         self,
         body_batch: Optional[List[Dict]] = None,
@@ -429,7 +430,7 @@ class Vespa(object):
 
     def _feed_batch_sync(
         self, schema: str, batch: List[Dict], namespace: str
-    ) -> VespaResponse:
+    ) -> List[VespaResponse]:
         return [
             self.feed_data_point(
                 schema, data_point["id"], data_point["fields"], namespace
@@ -447,7 +448,8 @@ class Vespa(object):
                 schema=schema, batch=batch, namespace=namespace
             )
 
-    def feed_batch(
+    @retry(wait=wait_exponential(multiplier=1), stop=stop_after_attempt(3))
+    def _feed_batch(
         self,
         batch: List[Dict],
         schema: Optional[str] = None,
@@ -493,16 +495,65 @@ class Vespa(object):
                 schema=schema, batch=batch, namespace=namespace
             )
 
-    def feed_df(self, df: DataFrame, include_id: bool = True, **kwargs):
+    def feed_batch(
+        self,
+        batch: List[Dict],
+        schema: Optional[str] = None,
+        asynchronous=True,
+        connections: Optional[int] = 100,
+        total_timeout: int = 100,
+        namespace: Optional[str] = None,
+        batch_size=1000,
+    ):
+        """
+        Feed a batch of data to a Vespa app.
+
+        :param batch: A list of dict containing the keys 'id' and 'fields' to be used in the :func:`feed_data_point`.
+        :param schema: The schema that we are sending data to. The schema is optional in case it is possible to infer
+            the schema from the application package.
+        :param asynchronous: Set True to send data in async mode. Default to True.
+        :param connections: Number of allowed concurrent connections, valid only if `asynchronous=True`.
+        :param total_timeout: Total timeout in secs for each of the concurrent requests when using `asynchronous=True`.
+        :param namespace: The namespace that we are sending data to. If no namespace is provided the schema is used.
+        :param batch_size: The number of documents to feed per batch.
+        :return: List of HTTP POST responses
+        """
+        mini_batches = [
+            batch[i : i + batch_size] for i in range(0, len(batch), batch_size)
+        ]
+        batch_http_responses = []
+        for idx, mini_batch in enumerate(mini_batches):
+            feed_results = self._feed_batch(
+                batch=mini_batch,
+                schema=schema,
+                asynchronous=asynchronous,
+                connections=connections,
+                total_timeout=total_timeout,
+                namespace=namespace,
+            )
+            batch_http_responses.extend(feed_results)
+            status_code_summary = Counter([x.status_code for x in feed_results])
+            print(
+                "Successful documents fed: {}/{}.\nBatch progress: {}/{}.".format(
+                    status_code_summary[200],
+                    len(mini_batch),
+                    idx + 1,
+                    len(mini_batches),
+                )
+            )
+        return batch_http_responses
+
+    def feed_df(self, df: DataFrame, include_id: bool = True, id_field="id", **kwargs):
         """
         Feed data contained in a DataFrame.
 
         :param df: A DataFrame containing a required 'id' column and the remaining fields to be fed.
         :param include_id: Include id on the fields to be fed. Default to True.
+        :param id_field: Name of the column containing the id field.
         :param kwargs: Additional parameters are passed to :func:`feed_batch`.
         :return: List of HTTP POST responses
         """
-        batch = parse_feed_df(df=df, include_id=include_id)
+        batch = parse_feed_df(df=df, include_id=include_id, id_field=id_field)
         return self.feed_batch(batch=batch, **kwargs)
 
     def delete_data(
@@ -855,7 +906,12 @@ class Vespa(object):
                     hits=number_additional_docs,
                     **kwargs,
                 )
-                hits.extend(random_hits_result.hits)
+                new_hits = [
+                    hit
+                    for hit in random_hits_result.hits
+                    if hit["fields"][id_field] != relevant_id
+                ]
+                hits.extend(new_hits)
 
             features = self.annotate_data(
                 hits=hits,
@@ -956,61 +1012,20 @@ class Vespa(object):
         training_data = DataFrame.from_records(training_data)
         return training_data
 
-    def evaluate_query(
-        self,
-        eval_metrics: List[EvalMetric],
-        query_model: QueryModel,
-        query_id: str,
-        query: str,
-        id_field: str,
-        relevant_docs: List[Dict],
-        default_score: int = 0,
-        detailed_metrics=False,
-        **kwargs,
-    ) -> Dict:
-        """
-        Evaluate a query according to evaluation metrics
-
-        :param eval_metrics: A list of evaluation metrics.
-        :param query_model: Query model.
-        :param query_id: Query id represented as str.
-        :param query: Query string.
-        :param id_field: The Vespa field representing the document id.
-        :param relevant_docs: A list with dicts where each dict contains a doc id a optionally a doc score.
-        :param default_score: Score to assign to the additional documents that are not relevant. Default to 0.
-        :param detailed_metrics: Return intermediate computations if available.
-        :param kwargs: Extra keyword arguments to be included in the Vespa Query.
-        :return: Dict containing query_id and metrics according to the selected evaluation metrics.
-        """
-
-        query_results = self.query(query=query, query_model=query_model, **kwargs)
-        evaluation = {"model": query_model.name, "query_id": query_id}
-        for evaluator in eval_metrics:
-            evaluation.update(
-                evaluator.evaluate_query(
-                    query_results,
-                    relevant_docs,
-                    id_field,
-                    default_score,
-                    detailed_metrics,
-                )
-            )
-        return evaluation
-
-    def evaluate(
+    def collect_vespa_features(
         self,
         labeled_data: Union[List[Dict], DataFrame],
-        eval_metrics: List[EvalMetric],
-        query_model: Union[QueryModel, List[QueryModel]],
         id_field: str,
+        query_model: QueryModel,
+        number_additional_docs: int,
+        fields: List[str],
+        keep_features: Optional[List[str]] = None,
+        relevant_score: int = 1,
         default_score: int = 0,
-        detailed_metrics=False,
-        per_query=False,
-        aggregators=None,
         **kwargs,
     ) -> DataFrame:
         """
-        Evaluate a :class:`QueryModel` according to a list of :class:`EvalMetric`.
+        Collect Vespa features based on a set of labelled data.
 
         labeled_data can be a DataFrame or a List of Dict:
 
@@ -1037,54 +1052,151 @@ class Vespa(object):
         ... ]
 
         :param labeled_data: Labelled data containing query, query_id and relevant ids. See details about data format.
-        :param eval_metrics: A list of evaluation metrics.
-        :param query_model: Accept a Query model or a list of Query Models.
         :param id_field: The Vespa field representing the document id.
+        :param query_model: Query model.
+        :param number_additional_docs: Number of additional documents to retrieve for each relevant document.
+        :param fields: List of Vespa fields to collect, e.g. ["rankfeatures", "summaryfeatures"]
+        :param keep_features: List containing the names of the features that should be returned. Default to None,
+            which return all the features contained in the 'fields' argument.
+        :param relevant_score: Score to assign to relevant documents. Default to 1.
         :param default_score: Score to assign to the additional documents that are not relevant. Default to 0.
-        :param detailed_metrics: Return intermediate computations if available.
-        :param per_query: Set to True to return evaluation metrics per query.
-        :param aggregators: Used only if `per_query=False`. List of pandas friendly aggregators to summarize per model
-            metrics. We use ["mean", "median", "std"] by default.
         :param kwargs: Extra keyword arguments to be included in the Vespa Query.
-        :return: DataFrame containing query_id and metrics according to the selected evaluation metrics.
+        :return: DataFrame containing document id (document_id), query id (query_id), scores (relevant)
+            and vespa rank features returned by the Query model RankProfile used.
         """
+
         if isinstance(labeled_data, DataFrame):
             labeled_data = parse_labeled_data(df=labeled_data)
 
-        if isinstance(query_model, QueryModel):
-            query_model = [query_model]
-
-        model_names = [model.name for model in query_model]
-        assert len(model_names) == len(
-            set(model_names)
-        ), "Duplicate model names. Choose unique model names."
-
-        evaluation = []
-        for query_data in labeled_data:
-            for model in query_model:
-                evaluation_query = self.evaluate_query(
-                    eval_metrics=eval_metrics,
-                    query_model=model,
-                    query_id=query_data["query_id"],
-                    query=query_data["query"],
-                    id_field=id_field,
-                    relevant_docs=query_data["relevant_docs"],
-                    default_score=default_score,
-                    detailed_metrics=detailed_metrics,
-                    **kwargs,
-                )
-                evaluation.append(evaluation_query)
-        evaluation = DataFrame.from_records(evaluation)
-        if not per_query:
-            if not aggregators:
-                aggregators = ["mean", "median", "std"]
-            evaluation = (
-                evaluation[[x for x in evaluation.columns if x != "query_id"]]
-                .groupby(by="model")
-                .agg(aggregators)
-                .T
+        flat_data = [
+            (
+                data["query_id"],
+                data["query"],
+                relevant_doc["id"],
+                relevant_doc.get("score", relevant_score),
             )
-        return evaluation
+            for data in labeled_data
+            for relevant_doc in data["relevant_docs"]
+        ]
+
+        queries = [x[1] for x in flat_data]
+        relevant_search = self.query_batch(
+            query_batch=queries,
+            query_model=query_model,
+            recall=[(id_field, [x[2]]) for x in flat_data],
+            **kwargs,
+        )
+        result = []
+        for ((query_id, query, relevant_id, relevant_score), query_result) in zip(
+            flat_data, relevant_search
+        ):
+            result.extend(
+                self.annotate_data(
+                    hits=query_result.hits,
+                    query_id=query_id,
+                    id_field=id_field,
+                    relevant_id=relevant_id,
+                    fields=fields,
+                    relevant_score=relevant_score,
+                    default_score=default_score,
+                )
+            )
+        if number_additional_docs > 0:
+            additional_hits_result = self.query_batch(
+                query_batch=queries,
+                query_model=query_model,
+                hits=number_additional_docs,
+                **kwargs,
+            )
+            for ((query_id, query, relevant_id, relevant_score), query_result) in zip(
+                flat_data, additional_hits_result
+            ):
+                result.extend(
+                    self.annotate_data(
+                        hits=query_result.hits,
+                        query_id=query_id,
+                        id_field=id_field,
+                        relevant_id=relevant_id,
+                        fields=fields,
+                        relevant_score=relevant_score,
+                        default_score=default_score,
+                    )
+                )
+        df = DataFrame.from_records(result)
+        df = df.drop_duplicates(["document_id", "query_id", "label"])
+        df = df.sort_values("query_id")
+        if keep_features:
+            df = df[["document_id", "query_id", "label"] + keep_features]
+        return df
+
+    def store_vespa_features(
+        self,
+        output_file_path: str,
+        labeled_data: Union[List[Dict], DataFrame],
+        id_field: str,
+        query_model: QueryModel,
+        number_additional_docs: int,
+        fields: List[str],
+        keep_features: Optional[List[str]] = None,
+        relevant_score: int = 1,
+        default_score: int = 0,
+        batch_size=1000,
+        **kwargs,
+    ):
+        """
+        Retrieve Vespa rank features and store them in a .csv file.
+
+        :param output_file_path: Path of the .csv output file. It will create the file of it does not exist and
+            append the vespa features to an pre-existing file.
+        :param labeled_data: Labelled data containing query, query_id and relevant ids. See details about data format.
+        :param id_field: The Vespa field representing the document id.
+        :param query_model: Query model.
+        :param number_additional_docs: Number of additional documents to retrieve for each relevant document.
+        :param fields: List of Vespa fields to collect, e.g. ["rankfeatures", "summaryfeatures"]
+        :param keep_features: List containing the names of the features that should be returned. Default to None,
+            which return all the features contained in the 'fields' argument.
+        :param relevant_score: Score to assign to relevant documents. Default to 1.
+        :param default_score: Score to assign to the additional documents that are not relevant. Default to 0.
+        :param batch_size: The size of the batch of labeled data points to be processed.
+        :param kwargs: Extra keyword arguments to be included in the Vespa Query.
+        :return: returns 0 upon success.
+        """
+
+        if isinstance(labeled_data, DataFrame):
+            labeled_data = parse_labeled_data(df=labeled_data)
+
+        mini_batches = [
+            labeled_data[i : i + batch_size]
+            for i in range(0, len(labeled_data), batch_size)
+        ]
+        for idx, mini_batch in enumerate(mini_batches):
+            vespa_features = self.collect_vespa_features(
+                labeled_data=mini_batch,
+                id_field=id_field,
+                query_model=query_model,
+                number_additional_docs=number_additional_docs,
+                fields=fields,
+                keep_features=keep_features,
+                relevant_score=relevant_score,
+                default_score=default_score,
+                **kwargs,
+            )
+            if os.path.isfile(output_file_path):
+                vespa_features.to_csv(
+                    path_or_buf=output_file_path, header=False, index=False, mode="a"
+                )
+            else:
+                vespa_features.to_csv(
+                    path_or_buf=output_file_path, header=True, index=False, mode="w"
+                )
+            print(
+                "Rows collected: {}.\nBatch progress: {}/{}.".format(
+                    vespa_features.shape[0],
+                    idx + 1,
+                    len(mini_batches),
+                )
+            )
+        return 0
 
     @property
     def application_package(self):
@@ -1119,7 +1231,7 @@ class Vespa(object):
         model = self.get_model_from_application_package(model_id)
         encoded_tokens = model.create_url_encoded_tokens(x=x)
         with VespaSync(self) as sync_app:
-            return TextTask.parse_vespa_prediction(
+            return model.parse_vespa_prediction(
                 sync_app.predict(
                     model_id=model_id,
                     function_name=function_name,
