@@ -1,9 +1,12 @@
 import os
+import zipfile
+
 from pathlib import Path
+from shutil import copyfile
 from typing import List, Mapping, Optional, Union, Dict
 from collections import OrderedDict
-
 from jinja2 import Environment, PackageLoader, select_autoescape
+from io import BytesIO
 
 from vespa.json_serialization import ToJson, FromJson
 from vespa.query import QueryModel
@@ -39,12 +42,11 @@ class HNSW(ToJson, FromJson["HNSW"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {
+        return {
             "distance_metric": self.distance_metric,
             "max_links_per_node": self.max_links_per_node,
             "neighbors_to_explore_at_insert": self.neighbors_to_explore_at_insert,
         }
-        return map
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -142,16 +144,16 @@ class Field(ToJson, FromJson["Field"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {"name": self.name, "type": self.type}
+        m = {"name": self.name, "type": self.type}
         if self.indexing is not None:
-            map.update(indexing=self.indexing)
+            m.update(indexing=self.indexing)
         if self.index is not None:
-            map.update(index=self.index)
+            m.update(index=self.index)
         if self.attribute is not None:
-            map.update(attribute=self.attribute)
+            m.update(attribute=self.attribute)
         if self.ann is not None:
-            map.update(ann=self.ann.to_envelope)
-        return map
+            m.update(ann=self.ann.to_envelope)
+        return m
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -216,12 +218,11 @@ class ImportedField(ToJson, FromJson["ImportedField"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {
+        return {
             "name": self.name,
             "reference_field": self.reference_field,
             "field_to_import": self.field_to_import,
         }
-        return map
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -294,11 +295,10 @@ class Document(ToJson, FromJson["Document"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {
+        return {
             "fields": [field.to_envelope for field in self.fields],
             "inherits": self.inherits,
         }
-        return map
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -342,8 +342,7 @@ class FieldSet(ToJson, FromJson["FieldSet"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {"name": self.name, "fields": self.fields}
-        return map
+        return {"name": self.name, "fields": self.fields}
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -412,8 +411,7 @@ class Function(ToJson, FromJson["Function"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {"name": self.name, "expression": self.expression, "args": self.args}
-        return map
+        return {"name": self.name, "expression": self.expression, "args": self.args}
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -461,8 +459,7 @@ class SecondPhaseRanking(ToJson, FromJson["SecondPhaseRanking"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {"expression": self.expression, "rerank_count": self.rerank_count}
-        return map
+        return {"expression": self.expression, "rerank_count": self.rerank_count}
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -581,22 +578,22 @@ class RankProfile(ToJson, FromJson["RankProfile"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {
+        m = {
             "name": self.name,
             "first_phase": self.first_phase,
         }
         if self.inherits is not None:
-            map.update({"inherits": self.inherits})
+            m.update({"inherits": self.inherits})
         if self.constants is not None:
-            map.update({"constants": self.constants})
+            m.update({"constants": self.constants})
         if self.functions is not None:
-            map.update({"functions": [f.to_envelope for f in self.functions]})
+            m.update({"functions": [f.to_envelope for f in self.functions]})
         if self.summary_features is not None:
-            map.update({"summary_features": self.summary_features})
+            m.update({"summary_features": self.summary_features})
         if self.second_phase is not None:
-            map.update({"second_phase": self.second_phase.to_envelope})
+            m.update({"second_phase": self.second_phase.to_envelope})
 
-        return map
+        return m
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -679,13 +676,12 @@ class OnnxModel(ToJson, FromJson["OnnxModel"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {
+        return {
             "model_name": self.model_name,
             "model_file_path": self.model_file_path,
             "inputs": self.inputs,
             "outputs": self.outputs,
         }
-        return map
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -855,28 +851,28 @@ class Schema(ToJson, FromJson["Schema"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {
+        m = {
             "name": self.name,
             "document": self.document.to_envelope,
             "global_document": self.global_document,
         }
         if self.fieldsets:
-            map["fieldsets"] = [
+            m["fieldsets"] = [
                 self.fieldsets[name].to_envelope for name in self.fieldsets.keys()
             ]
         if self.rank_profiles:
-            map["rank_profiles"] = [
+            m["rank_profiles"] = [
                 self.rank_profiles[name].to_envelope
                 for name in self.rank_profiles.keys()
             ]
         if self.models:
-            map["models"] = [model.to_envelope for model in self.models]
+            m["models"] = [model.to_envelope for model in self.models]
         if self.imported_fields:
-            map["imported_fields"] = [
+            m["imported_fields"] = [
                 self.imported_fields[name].to_envelope
                 for name in self.imported_fields.keys()
             ]
-        return map
+        return m
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -944,8 +940,7 @@ class QueryTypeField(ToJson, FromJson["QueryTypeField"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {"name": self.name, "type": self.type}
-        return map
+        return {"name": self.name, "type": self.type}
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -1015,8 +1010,7 @@ class QueryProfileType(ToJson, FromJson["QueryProfileType"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {"fields": [field.to_envelope for field in self.fields]}
-        return map
+        return {"fields": [field.to_envelope for field in self.fields]}
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -1056,8 +1050,7 @@ class QueryField(ToJson, FromJson["QueryField"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {"name": self.name, "value": self.value}
-        return map
+        return {"name": self.name, "value": self.value}
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -1113,8 +1106,7 @@ class QueryProfile(ToJson, FromJson["QueryProfile"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {"fields": [field.to_envelope for field in self.fields]}
-        return map
+        return {"fields": [field.to_envelope for field in self.fields]}
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -1171,19 +1163,16 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
         default_query_model: Optional[QueryModel] = None
     ) -> None:
         """
-        Create a Vespa Application Package.
-
-        Check the `Vespa documentation <https://docs.vespa.ai/en/cloudconfig/application-packages.html>`__
-        for more detailed information about application packages.
-
+        Create an `Application Package <https://docs.vespa.ai/en/cloudconfig/application-packages.html>`__.
         An :class:`ApplicationPackage` instance comes with a default :class:`Schema`
         that contains a default :class:`Document`
 
-        :param name: Application name.
-        :param schema: List of :class:`Schema`s of the application. If `None`, an empty :class:`Schema` with the same name of the
-            application will be created by default.
-        :param query_profile: :class:`QueryProfile` of the application. If `None`, a :class:`QueryProfile` named `default`
-         with :class:`QueryProfileType` named `root` will be created by default.
+        :param name: Application name. Cannot contain '-' or '_'.
+        :param schema: List of :class:`Schema`s of the application.
+            If `None`, an empty :class:`Schema` with the same name of the application will be created by default.
+        :param query_profile: :class:`QueryProfile` of the application.
+            If `None`, a :class:`QueryProfile` named `default` with :class:`QueryProfileType` named `root`
+            will be created by default.
         :param query_profile_type: :class:`QueryProfileType` of the application. If `None`, a empty
             :class:`QueryProfileType` named `root` will be created by default.
         :param stateless_model_evaluation: Enable stateless model evaluation. Default to False.
@@ -1196,12 +1185,14 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
 
         The easiest way to get started is to create a default application package:
 
-        >>> ApplicationPackage(name="test_app")
-        ApplicationPackage('test_app', [Schema('test_app', Document(None, None), None, None, [], False, None)], QueryProfile(None), QueryProfileType(None))
+        >>> ApplicationPackage(name="testapp")
+        ApplicationPackage('testapp', [Schema('testapp', Document(None, None), None, None, [], False, None)], QueryProfile(None), QueryProfileType(None))
 
         It will create a default :class:`Schema`, :class:`QueryProfile` and :class:`QueryProfileType` that you can then
         populate with specifics of your application.
         """
+        if not name.isalnum():
+            raise ValueError("Application package name can only contain [a-zA-Z0-9], was '{}'".format(name))
         self.name = name
         if not schema:
             schema = (
@@ -1348,21 +1339,6 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
             query_profile_type=self.query_profile_type
         )
 
-    @property
-    def hosts_to_text(self):
-        env = Environment(
-            loader=PackageLoader("vespa", "templates"),
-            autoescape=select_autoescape(
-                disabled_extensions=("txt",),
-                default_for_string=True,
-                default=True,
-            ),
-        )
-        env.trim_blocks = True
-        env.lstrip_blocks = True
-        schema_template = env.get_template("hosts.xml")
-        return schema_template.render()
-
     @property
     def services_to_text(self):
         env = Environment(
@@ -1391,10 +1367,10 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {"name": self.name}
+        m = {"name": self.name}
         if self._schema is not None:
-            map.update({"schema": [x.to_envelope for x in self.schemas]})
-        return map
+            m.update({"schema": [x.to_envelope for x in self.schemas]})
+        return m
 
     @staticmethod
     def _application_package_file_name(disk_folder):
@@ -1412,6 +1388,107 @@ class ApplicationPackage(ToJson, FromJson["ApplicationPackage"]):
         with open(file_path, "r") as f:
             return ApplicationPackage.from_json(f.read())
 
+    def to_zip(self) -> BytesIO:
+        """
+        Return the application package as zipped bytes,
+        to be used in a subsequent deploy
+        :return: BytesIO buffer
+        """
+        buffer = BytesIO()
+        with zipfile.ZipFile(buffer, "a") as zip_archive:
+            zip_archive.writestr(
+                "services.xml", self.services_to_text
+            )
+
+            for schema in self.schemas:
+                zip_archive.writestr(
+                    "schemas/{}.sd".format(schema.name),
+                    schema.schema_to_text,
+                )
+                for model in schema.models:
+                    zip_archive.write(
+                        model.model_file_path,
+                        "files/{}".format(model.model_file_name),
+                    )
+
+            if self.models:
+                for model_id, model in self.models.items():
+                    temp_model_file = "{}.onnx".format(model_id)
+                    model.export_to_onnx(output_path=temp_model_file)
+                    zip_archive.write(
+                        temp_model_file,
+                        "models/{}.onnx".format(model_id),
+                    )
+                    os.remove(temp_model_file)
+
+            if self.query_profile:
+                zip_archive.writestr(
+                    "search/query-profiles/default.xml",
+                    self.query_profile_to_text,
+                )
+                zip_archive.writestr(
+                    "search/query-profiles/types/root.xml",
+                    self.query_profile_type_to_text,
+                )
+
+        buffer.seek(0)
+        return buffer
+
+        # ToDo: use this for the Vespa Cloud app package
+        #zip_archive.writestr(
+        #    "application/security/clients.pem",
+        #    app.public_bytes(serialization.Encoding.PEM),
+        #)
+
+    def to_zipfile(self, zfile: Path) -> None:
+        """
+        Export the application package as a deployable zipfile.
+        See `application packages <https://docs.vespa.ai/en/cloudconfig/application-packages.html>`__
+        for deployment options.
+
+        :param zfile: Filename to export to
+        :return:
+        """
+        with open(zfile, "wb") as f:
+            f.write(self.to_zip().getbuffer().tobytes())
+
+    def to_files(self, root: Path) -> None:
+        """
+        Export the application package as a directory tree.
+
+        :param root: Directory to export files to
+        :return:
+        """
+        if not os.path.exists(root):
+            raise ValueError("Invalid path for export: {}".format(root))
+
+        Path(os.path.join(root, "schemas")).mkdir(parents=True, exist_ok=True)
+        Path(os.path.join(root, "files")).mkdir(parents=True, exist_ok=True)
+        Path(os.path.join(root, "models")).mkdir(parents=True, exist_ok=True)
+        Path(os.path.join(root, "search/query-profiles/types")).mkdir(parents=True, exist_ok=True)
+
+        for schema in self.schemas:
+            with open(os.path.join(root, "schemas/{}.sd".format(schema.name)), "w") as f:
+                f.write(schema.schema_to_text)
+            for model in schema.models:
+                copyfile(
+                    model.model_file_path,
+                    os.path.join(root, "files", model.model_file_name)
+                )
+
+        if self.query_profile:
+            with open(os.path.join(root, "search/query-profiles/default.xml"), "w") as f:
+                f.write(self.query_profile_to_text)
+            with open(os.path.join(root, "search/query-profiles/types/root.xml"), "w") as f:
+                f.write(self.query_profile_type_to_text)
+
+        with open(os.path.join(root, "services.xml"), "w") as f:
+            f.write(self.services_to_text)
+
+        if self.models:
+            for model_id, model in self.models.items():
+                model.export_to_onnx(output_path=os.path.join(root, "models/{}.onnx".format(model_id)))
+
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
             return False
@@ -1458,5 +1535,4 @@ class ModelServer(ApplicationPackage):
 
     @property
     def to_dict(self) -> Mapping:
-        map = {"name": self.name}
-        return map
+        return {"name": self.name}
