@@ -90,7 +90,7 @@ class TestVespaQuery(unittest.TestCase):
                 hits=10,
             ).request_body,
             {
-                "yql": 'select * from sources * where ([{"grammar": "any"}]userInput("this is a test"));',
+                "yql": 'select * from sources * where ({grammar: "any"}userInput("this is a test"));',
                 "ranking": {"profile": "default", "listFeatures": "false"},
                 "hits": 10,
             },
@@ -105,7 +105,7 @@ class TestVespaQuery(unittest.TestCase):
                 recall=("id", [1, 5]),
             ).request_body,
             {
-                "yql": 'select * from sources * where ([{"grammar": "any"}]userInput("this is a test"));',
+                "yql": 'select * from sources * where ({grammar: "any"}userInput("this is a test"));',
                 "ranking": {"profile": "default", "listFeatures": "false"},
                 "hits": 10,
                 "recall": "+(id:1 id:5)",
@@ -618,43 +618,3 @@ class TestVespaEvaluate(unittest.TestCase):
                 ],
             }
         }
-
-    def test_evaluate_query(self):
-        self.app.query = Mock(return_value={})
-        eval_metric = Mock()
-        eval_metric.evaluate_query = Mock(return_value={"metric": 1})
-        eval_metric2 = Mock()
-        eval_metric2.evaluate_query = Mock(return_value={"metric_2": 2})
-        query_model = QueryModel()
-        evaluation = self.app.evaluate_query(
-            eval_metrics=[eval_metric, eval_metric2],
-            query_model=query_model,
-            query_id="0",
-            query="this is a test",
-            id_field="vespa_id_field",
-            relevant_docs=self.labeled_data[0]["relevant_docs"],
-            default_score=0,
-            hits=10,
-        )
-        self.assertEqual(self.app.query.call_count, 1)
-        self.app.query.assert_has_calls(
-            [
-                call(query="this is a test", query_model=query_model, hits=10),
-            ]
-        )
-        self.assertEqual(eval_metric.evaluate_query.call_count, 1)
-        eval_metric.evaluate_query.assert_has_calls(
-            [
-                call(
-                    {},
-                    self.labeled_data[0]["relevant_docs"],
-                    "vespa_id_field",
-                    0,
-                    False,
-                ),
-            ]
-        )
-        self.assertDictEqual(
-            evaluation,
-            {"model": "default_name", "query_id": "0", "metric": 1, "metric_2": 2},
-        )
