@@ -4,7 +4,7 @@ import os
 import re
 import asyncio
 import json
-from pandas import DataFrame
+from pandas import DataFrame, read_csv
 from vespa.package import (
     HNSW,
     Document,
@@ -22,6 +22,8 @@ from vespa.query import QueryModel, RankProfile as Ranking, OR, QueryRankingFeat
 from vespa.gallery import QuestionAnswering, TextSearch
 from vespa.application import VespaSync
 
+CONTAINER_STOP_TIMEOUT = 600
+
 
 def create_msmarco_application_package():
     #
@@ -145,12 +147,11 @@ class TestDockerCommon(unittest.TestCase):
         # Test VespaDocker serialization
         #
         self.assertEqual(
-            repr(self.vespa_docker), repr(VespaDocker.from_dict(self.vespa_docker.to_dict))
+            repr(self.vespa_docker),
+            repr(VespaDocker.from_dict(self.vespa_docker.to_dict)),
         )
 
-    def create_vespa_docker_from_container_name_or_id(
-        self, application_package
-    ):
+    def create_vespa_docker_from_container_name_or_id(self, application_package):
         #
         # Raises ValueError if container does not exist
         #
@@ -169,13 +170,11 @@ class TestDockerCommon(unittest.TestCase):
     def redeploy_with_container_stopped(self, application_package):
         self.vespa_docker = VespaDocker(port=8089)
         self.vespa_docker.deploy(application_package=application_package)
-        self.vespa_docker.container.stop()
+        self.vespa_docker.container.stop(timeout=CONTAINER_STOP_TIMEOUT)
         app = self.vespa_docker.deploy(application_package=application_package)
         self.assertEqual(app.get_application_status().status_code, 200)
 
-    def redeploy_with_application_package_changes(
-        self, application_package
-    ):
+    def redeploy_with_application_package_changes(self, application_package):
         self.vespa_docker = VespaDocker(port=8089)
         app = self.vespa_docker.deploy(application_package=application_package)
         res = app.query(
@@ -186,7 +185,7 @@ class TestDockerCommon(unittest.TestCase):
         ).json
         self.assertIsNotNone(
             re.search(
-                "Requested rank profile 'new-rank-profile' is undefined for document type ",
+                r"schema[\s\S]+ does not contain requested rank profile",
                 res["root"]["errors"][0]["message"],
             )
         )
@@ -726,9 +725,7 @@ class TestApplicationCommon(unittest.TestCase):
         # Query data
         #
         if query_batch:
-            result = app.query_batch(
-                query_batch=query_batch, query_model=query_model
-            )
+            result = app.query_batch(query_batch=query_batch, query_model=query_model)
             for idx, first_hit in enumerate(queries_first_hit):
                 self.assertEqual(
                     first_hit, result[idx].hits[0]["fields"][hit_field_to_check]
@@ -913,7 +910,7 @@ class TestApplicationCommon(unittest.TestCase):
 
     @staticmethod
     def _parse_vespa_tensor(hit, feature):
-        return [x["value"] for x in hit["fields"]["summaryfeatures"][feature]["cells"]]
+        return hit["fields"]["summaryfeatures"][feature]["values"][0]
 
     def bert_model_input_and_output(
         self, app, schema_name, fields_to_send, model_config
@@ -946,14 +943,12 @@ class TestApplicationCommon(unittest.TestCase):
                 rank_profile=Ranking(name="pretrained_bert_tiny"),
             ),
         )
-        vespa_input_ids = self._parse_vespa_tensor(
-            result.hits[0], "rankingExpression(input_ids)"
-        )
+        vespa_input_ids = self._parse_vespa_tensor(result.hits[0], "input_ids")
         vespa_attention_mask = self._parse_vespa_tensor(
-            result.hits[0], "rankingExpression(attention_mask)"
+            result.hits[0], "attention_mask"
         )
         vespa_token_type_ids = self._parse_vespa_tensor(
-            result.hits[0], "rankingExpression(token_type_ids)"
+            result.hits[0], "token_type_ids"
         )
 
         expected_inputs = model_config.create_encodings(
@@ -967,12 +962,12 @@ class TestApplicationCommon(unittest.TestCase):
             queries=["this is a test"], docs=[fields_to_send["title"]]
         )
         self.assertAlmostEqual(
-            result.hits[0]["fields"]["summaryfeatures"]["rankingExpression(logit0)"],
+            result.hits[0]["fields"]["summaryfeatures"]["logit0"],
             expected_logits[0][0],
             5,
         )
         self.assertAlmostEqual(
-            result.hits[0]["fields"]["summaryfeatures"]["rankingExpression(logit1)"],
+            result.hits[0]["fields"]["summaryfeatures"]["logit1"],
             expected_logits[0][1],
             5,
         )
@@ -986,20 +981,26 @@ class TestMsmarcoDockerDeployment(TestDockerCommon):
         self.deploy(application_package=self.app_package)
 
     def test_instantiate_vespa_docker_from_container_name_or_id(self):
-        self.create_vespa_docker_from_container_name_or_id(application_package=self.app_package)
+        self.create_vespa_docker_from_container_name_or_id(
+            application_package=self.app_package
+        )
 
     @pytest.mark.skip(reason="Works locally but fails on Screwdriver")
     def test_redeploy_with_container_stopped(self):
         self.redeploy_with_container_stopped(application_package=self.app_package)
 
     def test_redeploy_with_application_package_changes(self):
-        self.redeploy_with_application_package_changes(application_package=self.app_package)
+        self.redeploy_with_application_package_changes(
+            application_package=self.app_package
+        )
 
     def test_trigger_start_stop_and_restart_services(self):
-        self.trigger_start_stop_and_restart_services(application_package=self.app_package)
+        self.trigger_start_stop_and_restart_services(
+            application_package=self.app_package
+        )
 
     def tearDown(self) -> None:
-        self.vespa_docker.container.stop()
+        self.vespa_docker.container.stop(timeout=CONTAINER_STOP_TIMEOUT)
         self.vespa_docker.container.remove()
 
 
@@ -1011,7 +1012,7 @@ class TestCord19DockerDeployment(TestDockerCommon):
         self.deploy(application_package=self.app_package)
 
     def tearDown(self) -> None:
-        self.vespa_docker.container.stop()
+        self.vespa_docker.container.stop(timeout=CONTAINER_STOP_TIMEOUT)
         self.vespa_docker.container.remove()
 
 
@@ -1021,13 +1022,15 @@ class TestQaDockerDeployment(TestDockerCommon):
 
     def test_deploy(self):
         self.deploy(application_package=self.app_package)
-        self.vespa_docker.container.stop()
+        self.vespa_docker.container.stop(timeout=CONTAINER_STOP_TIMEOUT)
         self.vespa_docker.container.remove()
 
     def test_deploy_image(self):
-        self.deploy(application_package=self.app_package,
-                    container_image="vespaengine/vespa:7.566.21")
-        self.vespa_docker.container.stop()
+        self.deploy(
+            application_package=self.app_package,
+            container_image="vespaengine/vespa:7.566.21",
+        )
+        self.vespa_docker.container.stop(timeout=CONTAINER_STOP_TIMEOUT)
         self.vespa_docker.container.remove()
 
 
@@ -1124,7 +1127,7 @@ class TestMsmarcoApplication(TestApplicationCommon):
         )
 
     def tearDown(self) -> None:
-        self.vespa_docker.container.stop()
+        self.vespa_docker.container.stop(timeout=CONTAINER_STOP_TIMEOUT)
         self.vespa_docker.container.remove()
 
 
@@ -1155,13 +1158,8 @@ class TestCord19Application(TestApplicationCommon):
             expected_fields.update(
                 {
                     "pretrained_bert_tiny_doc_token_ids": {
-                        "cells": [
-                            {
-                                "address": {"d0": str(x)},
-                                "value": float(tensor_field_values[x]),
-                            }
-                            for x in range(len(tensor_field_values))
-                        ]
+                        "type": f"tensor<float>(d0[{len(tensor_field_values)}])",
+                        "values": tensor_field_values,
                     }
                 }
             )
@@ -1174,6 +1172,77 @@ class TestCord19Application(TestApplicationCommon):
             for i in range(10)
         ]
 
+    def test_check_duplicated_features(self):
+        schema = "cord19"
+        docs = [
+            {"id": fields["id"], "fields": fields} for fields in self.fields_to_send
+        ]
+        self.app.feed_batch(
+            schema=schema,
+            batch=docs,
+            asynchronous=True,
+            connections=120,
+            total_timeout=50,
+        )
+        data = self.app.collect_training_data_point(
+            query="give me title 1",
+            query_id="0",
+            relevant_id="1",
+            id_field="id",
+            query_model=QueryModel(
+                match_phase=OR(), rank_profile=Ranking(name="bm25", list_features=True)
+            ),
+            number_additional_docs=10,
+            fields=["rankfeatures", "summaryfeatures"],
+        )
+        document_ids = [x["document_id"] for x in data]
+        self.assertEqual(len(document_ids), len(set(document_ids)))
+
+    def test_store_vespa_features(self):
+        schema = "cord19"
+        docs = [
+            {"id": fields["id"], "fields": fields} for fields in self.fields_to_send
+        ]
+        self.app.feed_batch(
+            schema=schema,
+            batch=docs,
+            asynchronous=True,
+            connections=120,
+            total_timeout=50,
+        )
+        labeled_data = [
+            {
+                "query_id": 0,
+                "query": "give me title 1",
+                "relevant_docs": [{"id": "1", "score": 1}],
+            },
+            {
+                "query_id": 1,
+                "query": "give me title 3",
+                "relevant_docs": [{"id": "3", "score": 1}],
+            },
+        ]
+
+        self.app.store_vespa_features(
+            output_file_path=os.path.join(
+                os.environ["RESOURCES_DIR"], "vespa_features.csv"
+            ),
+            labeled_data=labeled_data,
+            id_field="id",
+            query_model=QueryModel(
+                match_phase=OR(), rank_profile=Ranking(name="bm25", list_features=True)
+            ),
+            number_additional_docs=2,
+            fields=["rankfeatures", "summaryfeatures"],
+        )
+        rank_features = read_csv(
+            os.path.join(os.environ["RESOURCES_DIR"], "vespa_features.csv")
+        )
+        # at least two relevant docs
+        self.assertTrue(rank_features.shape[0] > 2)
+        # at least one feature besides document_id, query_id and label
+        self.assertTrue(rank_features.shape[1] > 3)
+
     def test_model_endpoints_when_no_model_is_available(self):
         self.get_model_endpoints_when_no_model_is_available(
             app=self.app,
@@ -1243,8 +1312,12 @@ class TestCord19Application(TestApplicationCommon):
         )
 
     def tearDown(self) -> None:
-        self.vespa_docker.container.stop()
+        self.vespa_docker.container.stop(timeout=CONTAINER_STOP_TIMEOUT)
         self.vespa_docker.container.remove()
+        try:
+            os.remove(os.path.join(os.environ["RESOURCES_DIR"], "vespa_features.csv"))
+        except OSError:
+            pass
 
 
 class TestQaApplication(TestApplicationCommon):
@@ -1272,10 +1345,8 @@ class TestQaApplication(TestApplicationCommon):
                 "dataset": d["dataset"],
                 "context_id": d["context_id"],
                 "sentence_embedding": {
-                    "cells": [
-                        {"address": {"x": str(idx)}, "value": value}
-                        for idx, value in enumerate(d["sentence_embedding"]["values"])
-                    ]
+                    "type": f"tensor<float>(x[{len(d['sentence_embedding']['values'])}])",
+                    "values": d["sentence_embedding"]["values"],
                 },
             }
             if len(d["questions"]) > 0:
@@ -1353,7 +1424,7 @@ class TestQaApplication(TestApplicationCommon):
         )
 
     def tearDown(self) -> None:
-        self.vespa_docker.container.stop()
+        self.vespa_docker.container.stop(timeout=CONTAINER_STOP_TIMEOUT)
         self.vespa_docker.container.remove()
 
 
@@ -1401,7 +1472,7 @@ class TestGalleryTextSearch(unittest.TestCase):
             self.assertIn("fields", hit)
 
     def tearDown(self) -> None:
-        self.vespa_docker.container.stop()
+        self.vespa_docker.container.stop(timeout=CONTAINER_STOP_TIMEOUT)
         self.vespa_docker.container.remove()
 
 
@@ -1423,5 +1494,5 @@ class TestSequenceClassification(TestApplicationCommon):
         )
 
     def tearDown(self) -> None:
-        self.vespa_docker.container.stop()
+        self.vespa_docker.container.stop(timeout=CONTAINER_STOP_TIMEOUT)
         self.vespa_docker.container.remove()
